from libcpp.set cimport set
from libcpp.map cimport map
from libcpp.string cimport string
from libcpp.vector cimport vector

cdef extern from "tfrecord_lite.h":
    cdef struct decoder_options:
        set[string] names;

    cdef struct decoder_output:
        vector[string] names
        map[string, vector[string]] bytes_features
        map[string, string] int64_features
        map[string, string] float_features

    decoder_output decode_tfrecord_example(const unsigned char *bytes, size_t length, const decoder_options &options)


cdef decode_example_internal(bytes buffer, vector[string] names):
    cdef decoder_options options
    options.names = set[string]()
    for name in names:
        options.names.insert(name)

    cdef decoder_output decoded = decode_tfrecord_example(buffer, len(buffer), options)
    result = {}

    import numpy as np

    for name in decoded.names:
        if decoded.int64_features.count(name) > 0:
            result[name] = np.frombuffer(decoded.int64_features[name], dtype=np.int64)
        elif decoded.float_features.count(name) > 0:
            result[name] = np.frombuffer(decoded.float_features[name], dtype=np.float32)
        elif decoded.bytes_features.count(name) > 0:
            result[name] = decoded.bytes_features[name]
        else:
            result[name] = []

    return result


def decode_example(bytes buffer, names=[]):
    decoded = decode_example_internal(buffer, [name.encode('utf-8') for name in names])
    return {key.decode('utf-8'): value for key, value in decoded.items()}



import os
import sys
from setuptools import setup, Extension
from Cython.Build import cythonize

extra_compile_args = []

if 'linux' in sys.platform or 'darwin' in sys.platform:
    extra_compile_args += ['-O3', '-Wall', '-std=c++11']

if 'darwin' in sys.platform:
    extra_compile_args += ['-stdlib=libc++']

setup(
    name = 'tfrecord_lite',
    version = '0.0.5',
    description = 'A lightweight tfrecord parser',
    url = 'https://github.com/jongwook/tfrecord_lite',
    author = 'Jong Wook Kim',
    author_email = 'jongwook@nyu.edu',
    license = 'MIT',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    keywords = 'tfrecord',
    project_urls = {
        'Source': 'https://github.com/jongwook/tfrecord_lite',
        'Tracker': 'https://github.com/jongwook/tfrecord_lite/issues'
    },
    install_requires = ['Cython', 'numpy'],
    python_requires = '>=3.6',
    package_data = {
        'tfrecord_lite': ['src/*', 'tfrecrod_lite.pyx']
    },
    ext_modules = cythonize(Extension(
        'tfrecord_lite',
        sources=[
            'tfrecord_lite.pyx',
            'src/tfrecord_lite.cpp',
            'src/example.pb.cpp',
            'src/feature.pb.cpp',
            'src/pb_common.cpp',
            'src/pb_decode.cpp',
            'src/pb_encode.cpp'],
        extra_compile_args=extra_compile_args,
        language='c++',
        include_dirs=['src/']
    ))
)
